###############################################################################
## This python script prepares the HDF5 data for the VSF analysis by         ##
## extracting the relevant data and save them in seperate FITS file that are ##
## the basis for the further calculations of relative velocities.            ##
###############################################################################


# import required python packages
import yt
from glob import glob
import numpy as np
from astropy.io import fits
from astropy import units as u
from astropy import constants as cont


# define source folder
dsrc   = '../../src_data/' # contains folders with simulation snapshots

# define folder that will store fits files
dfits  = dsrc + 'fits/'

# name tags of clouds
fcloud = ['M3','M4','M8']

# properties of clouds and cubes
dcloud  = 1.7e-22			# g cm-3; mass density corresponding to 100 cm-3
rad     = 20.				# pc; physical readius of clouds
nx      = 400				# box edge size of data cube; total volume = nx**3
ppc     = cont.pc.to('cm').value	# cm; 1 parsec
dgw     = 2*rad/nx * ppc		# cm; edge length
dens    = np.zeros([nx,nx,nx])


# list of quantities provided by the simualtion
# all possible data:
#what  = [ 'dens','temp','velocity_x','velocity_y','velocity_z','sound_speed','pres','eint','magx','magy','magz','magnetic_energy','gpot' ]
#quant = [ 'total volume density','gas temperature','x velocity','y velocity','z velocity','sound speed','gas pressure','internal energy density','x magnetic field','y magnetic field','z magnetic field','magnetic energy density','gravitational potential' ]
#unit  = [ 'g cm-3','K','km s-1','km s-1','km s-1','km s-1','g cm-1 s-2','cm2 s-2','G','G','G','erg cm-3','cm2 s-2' ]
#label = [ 'dens','tgas','vx','vy','vz','cs','pres','eint','magx','magy','magz','emag','gpot' ]

# data required for Chira et al. (prep)
what  = [ 'dens','velocity_x','velocity_y','velocity_z' ]
quant = [ 'total volume density','x velocity','y velocity','z velocity' ]
unit  = [ 'g cm-3','km s-1','km s-1','km s-1' ]
label = [ 'dens','vx','vy','vz' ]


# prepare fits files
for cloud in fcloud:
	# define physical centre of the cloud
	if(cloud == fcloud[0]): c = np.array([ 460. , -367. ,  0. ])*ppc
	if(cloud == fcloud[1]): c = np.array([ 177. ,  -30. ,  0. ])*ppc
	if(cloud == fcloud[2]): c = np.array([  70. ,  355. , 25. ])*ppc
	#define left and right edge of physical cube
	le = c - rad*ppc
	re = c + rad*ppc

	# collect list of snapsnots available for cloud
	ffiles = glob('%s%s*/Stra*' % (dsrc,cloud))
	ffiles.sort()

	for ifile in ffiles:
		print(ifile)
		#load data from snapshot
		fil3d  = yt.load(ifile)
		#fil3d.field_list	# prints info about data
		#fil3d.field_info	# prints info about data
		box    = fil3d.box(le,re)	# defines area of interest
		bcloud = fil3d.covering_grid(9,le,dims=[nx,nx,nx])	# defines box of interest for data extraction

		# extract data
		for iquant in range(len(quant)):
			ctmp = bcloud[what[iquant]].value
			if(unit[iquant] == 'km s-1'): ctmp = ctmp * u.cm.to('km') # convert velocity data from cm/s to km/s

			# write fits files (all data)
			ffits = fits.PrimaryHDU()
			ffits.header['quatity'] = quant[iquant]
			ffits.header['unit'] = unit[iquant]
			ffits.data = ctmp
			ffits.writeto('%s%s/%s_%s_%s.fits.bz2' % (dfits,label[iquant],cloud,label[iquant],ifile[73:77]),clobber=True)

			# write fits files (cloud-only)
			if(iquant == 0): 
				dens = ctmp
				hism = np.where(dens < dcloud)
			ctmp[hism] = 0.		# setting all non-cloud cells to 0.
			ffits = fits.PrimaryHDU()
			ffits.header['quatity'] = quant[iquant]
			ffits.header['unit'] = unit[iquant]
			ffits.data = ctmp
			ffits.writeto('%s%s/%s_%s_%s_ncloud100.fits.bz2' % (dfits,label[iquant],cloud,label[iquant],ifile[73:77]),clobber=True)
			



